<?php

class Democracy {

	var $id;
	var $poll;
	var $logIPs;
	var $hasVoted;
	
	var $question;
	var $allow_users_to_add;
	var $active;
	var $timestamp;
	
	var $votedFor;
	

	function Democracy($id = 0, $before_title = false, $after_title = false)
	{

		global $wpdb, $table_prefix;
		if ($id == 0)
			$poll = $wpdb->get_row("SELECT * FROM {$table_prefix}democracyQ WHERE current = 1 LIMIT 1");			
		else
			$poll = $wpdb->get_row("SELECT * FROM {$table_prefix}democracyQ WHERE id = {$id} LIMIT 1");			

        if ($poll)
        {
			$this->id = (int) $poll->id;
			
			 // if user's code specified no id, then we're working with the current poll
			$this->isCurrent = ($id == 0);
			
			$this->logIPs = (bool) get_option('democracy_logIPs');
			$this->hasVoted = $this->hasVoted();
			
			// use isset in case it reverted to a SQL query
			if ($this->hasVoted && isset($_COOKIE['demVoted_'.$this->id]))
			   $this->votedFor = $_COOKIE['demVoted_'.$this->id];
			else
			   $this->votedFor = false;
			
			$this->before_title = ($before_title) ? $before_title : stripslashes(get_option('democracy_before_title'));
			$this->after_title  = ($after_title) ? $after_title : stripslashes(get_option('democracy_after_title'));
			
			$this->question = stripslashes($poll->question);
			$this->allow_users_to_add = (bool) $poll->allowusers;
			$this->active = (bool) $poll->active;
			$this->added = (int) $poll->added;
        }

        
        // there is no poll with 'current' status
        else
        {
            $this->id = false;
            _e("<!-- Democracy has no active polls -->", 'democracy');
        }

	}
			
	function hasVoted ()
	{
	    if (!$this->id) return false;
        global $wpdb, $table_prefix;
        
        if (isset($_COOKIE['demVoted_'.$this->id]))
            return true;
            
        if ($this->logIPs)
        {
            $user_ip = ip2long($_SERVER['REMOTE_ADDR']);
            return $wpdb->query("SELECT ip FROM {$table_prefix}democracyIP WHERE qid = {$this->id} AND ip = {$user_ip} LIMIT 1");
        }
        
        return false;
	}
	
	function getAnswers()
	{
	    if (!$this->id) return false;

		global $wpdb, $table_prefix;
		
		$order = (get_option('democracy_order_answers')) ? ' ORDER BY votes DESC' : '';
		return $wpdb->get_results("SELECT * FROM {$table_prefix}democracyA WHERE qid = {$this->id}{$order}");	
	}

	// displays the vote interface of a poll
	function display ($showVoteScreen = true)
	{
	    if (!$this->id) return false;
		$output = '';

		$output .= "\n\t<div class='democracy'>";

		    $output .=  "\n\t\t{$this->before_title}{$this->question}{$this->after_title}";

			$output .=  "\n\t\t<div class='dem-results'>";
			if ($showVoteScreen && !$this->hasVoted)
				$output .= $this->showVoteScreen();
			else
				$output .= $this->showResults();
			$output .=  "\n\t\t</div>";
		
		
		$output .=  "\n\t</div>";
	
		return $output;
	}
	
	function showVoteScreen()
	{
	    if (!$this->id) return false;

		$output = '';

		$answers = $this->getAnswers();

		if ($answers)
		{
		
		
			$output .= "\n\t\t<form action='".get_bloginfo('wpurl')."/wp-content/plugins/democracy/democracy.php' onsubmit='return dem_Vote(this)'>";
		
			$output .= "\n\t\t<ul>";
			
			foreach ($answers as $answer)
			{
			    $word = stripslashes($answer->answer);
			
			    $output .= "\n\t\t\t<li>";
			    $output .= "\n\t\t\t\t\t<input type='radio' id='dem-choice-{$answer->aid}' value='{$answer->aid}' name='dem_poll_{$this->id}' />";
			    $output .= "\n\t\t\t\t\t<label for='dem-choice-{$answer->aid}'>{$word}</label>";
			    $output .= "\n\t\t\t</li>";
			}
			
			if ($this->allow_users_to_add)
			{
				if (isset($_GET['dem_add_user_answer']) && isset($_GET['dem_poll_id']) && $_GET['dem_poll_id'] == $this->id)
				{
					$output .= "\n\t\t\t<li>";
					$output .= "\n\t\t\t\t<input type='radio' name='dem_poll_{$this->id}' value='newAnswer' checked='checked' />";
					$output .= "\n\t\t\t\t<input type='text' name='dem_new_answer' />";
					$output .= "\n\t\t\t</li>";
				}
				
				else
				{
					$url = add_query_arg(array('dem_add_user_answer' => 'true', 'dem_poll_id' => $this->id));
					$output .= "\n\t\t\t<li>";
					$output .= " <a href='{$url}' rel='nofollow' onclick='return dem_addAnswer(this)' class='dem-add-answer'>".__('Add an Answer', 'democracy').'</a>';
					$output .= "\n\t\t\t</li>";
				}
			
			}
			
			$output .= "\n\t\t</ul>";
			
			$action = ($this->allow_users_to_add && isset($_GET['dem_add_user_answer']) && isset($_GET['dem_poll_id']) && $_GET['dem_poll_id'] == $this->id) ? 'add_answer' : 'vote';
			
			$output .= "\n\t\t\t<input type='hidden' name='dem_poll_id' value='{$this->id}' />";
			$output .= "\n\t\t\t<input type='hidden' name='dem_action' value='{$action}' />";
			$output .= "\n\t\t\t<input type='submit' class='dem-vote-button' value='".__('Vote', 'democracy')."' />";
			
			$url   = htmlspecialchars(add_query_arg(array('dem_action' => 'view', 'dem_poll_id' => $this->id)));
			$jsurl = get_bloginfo('wpurl').'/wp-content/plugins/democracy/democracy.php?dem_action=view&amp;dem_poll_id='.$this->id;
			
			$output .= "\n\t\t\t<a href='{$url}' onclick='return dem_getVotes(\"{$jsurl}\", this)' rel='nofollow' class='dem-vote-link'>".__('View Results', 'democracy').'</a>';
			
			$output .= "\n\t\t</form>";
			
		}
		
		return $output;

	}
	
	function showResults ($voted_for_aid = -1)
	{
	    if (!$this->id) return false;

		$output = '';

		$answers = $this->getAnswers();

		$max = 0;
		$total = 0;
		foreach ($answers as $answer)
		{
			$total += $answer->votes;
			if ($max < $answer->votes)
				$max = $answer->votes;
		}

		$output .= "\n\t\t<ul>";
				
		$from_total = get_option('democracy_graph_from_total');
		
		// only show the 'a guest has added an answer' text if it's happened..
		$has_added_by = false;
		
		foreach ($answers as $answer)
		{
		    $word = stripslashes($answer->answer);
		    $output .= "\n\t\t\t<li>";
		    $output .= "\n\t\t\t\t{$word}";
		    
		    if ($answer->added_by)
		    {
		    	$has_added_by = true;
		    	$output .= "<sup title='".__('Added by a guest', 'democracy')."'>1</sup>";
		    }
		    
		    $percent = ($answer->votes > 0) ? round($answer->votes / $total * 100) : 0;

		    $graph_percent = (!$from_total && $percent != 0) ? round($answer->votes / $max * 100) : $percent;
		    
		    // if it's a 100% or 0% bar, don't make it look like a double border 
		    $class = ($max == $answer->votes || $answer->votes == 0) ? 'dem-graph dem-winner' : 'dem-graph';
		    
		    if ($this->hasVoted && $answer->aid == $this->votedFor)
		      $class .= ' dem-voted-for-this';
		      

		    $output .= "\n\t\t\t\t<div class='{$class}'>";
		    	$output .= "<a style='width: {$graph_percent}%' href='#' onclick='return false'>";
		    	$output .= "<b>{$answer->votes}</b>";
		    	$output .= "<i>".sprintf(__("%s%% of all votes",'democracy'), $percent)."</i>";
		    	$output .= "</a>";
		    $output .= "</div>";
		    
		    $output .= "\n\t\t\t</li>";
		
		}

		$output .= "\n\t\t</ul>";
		$output .= "\n\t\t<em class='dem-total-votes'>".sprintf(__('Total Votes: %s', 'democracy'), $total).'</em>';
		$output .= "\n\t\t<em class='dem-total-votes'>".__('Started: ').date(get_settings('date_format'), $this->added).'</em>';

		if ($has_added_by)
			$output .= "\n\t\t<small class='dem-user-added'>".__('<sup>1</sup> = Added by a guest').'</small>';

		if (!$this->hasVoted && $this->active)
		{
			$url   = htmlspecialchars(add_query_arg(array('dem_action' => 'show_vote_screen', 'dem_poll_id' => $this->id)));
			$jsurl = get_bloginfo('wpurl').'/wp-content/plugins/democracy/democracy.php?dem_action=show_vote_screen&amp;dem_poll_id='.$this->id;

			$output .= "\n\t\t<a href='{$url}' class='dem-vote-link' rel='nofollow' onclick='return dem_getVotes(\"{$jsurl}\", this)'>".__('Back to Vote Screen', 'democracy').'</a>';
		}
		
		return $output;
	}	
	
	function addVote ($answer)
	{
	    if (!$this->id) return false;

		global $wpdb, $table_prefix;

		// only shown if they're manipulating URLs
		if ($this->hasVoted)
			return false;
		
		$answer = (int) $answer;
    	$wpdb->query("UPDATE {$table_prefix}democracyA SET votes = (votes+1) WHERE qid = {$this->id} AND aid = {$answer} LIMIT 1");

		$cookie_last = get_option('democracy_cookie_days') * 24 * 60 * 60;
	    setcookie("demVoted_{$this->id}", $answer,  time()+$cookie_last, COOKIEPATH);

		$this->hasVoted = true;
		$this->votedFor = $answer;

		if ($this->logIPs)
			$this->logIP();
	}
	
	function logIP()
	{
	    if (!$this->id) return false;

		global $wpdb, $table_prefix;
		
		$user_ip = ip2long($_SERVER['REMOTE_ADDR']);
		$wpdb->query("INSERT INTO {$table_prefix}democracyIP VALUES ({$user_ip}, {$this->id})");
	}
	
	function addInlineAnswer($answer)
	{
	    if (!$this->id) return false;
		if ($this->hasVoted) return false;

		global $wpdb, $table_prefix;
		
		$new_answer = $wpdb->escape(htmlspecialchars($answer));
		
		$wpdb->query("INSERT INTO {$table_prefix}democracyA VALUES ('', {$this->id}, '{$new_answer}', 0, 1)");
	    
	    return $wpdb->insert_id;
	}
	
	
}