<?php
/*
Plugin Name: Democracy
Plugin URI: http://blog.jalenack.com/archives/democracy/
Description: This poll plugin is easy to use, install, and manage. It uses AJAX to load data, making it quick and fun to use. Add <code>{democracy:*id*}</code> to your posts to include polls in your posts, where <code>*id*</code> corresponds to the ID of your poll.
Version: 2.0.1
Author: Andrew Sutherland
Author URI: http://blog.jalenack.com/
*/
$jal_dem_version = "2.0.1";
// get the democracy class
require_once('class.php');

// make sure WP is loaded, for ajax calls to this file...
$ajax = false;
if (!function_exists('add_action'))
{
	$ajax = true;
	require_once("../../../wp-config.php");
}

// wrapper for showing poll
function jal_democracy($id = 0, $before_title = false, $after_title = false)
{
	
	$democracy = new Democracy($id, $before_title, $after_title);
	
	$showVoteScreen = !(isset($_GET['dem_action']) && $_GET['dem_action'] == 'view' && ($_GET['dem_poll_id'] == $id || $democracy->isCurrent));
	echo $democracy->display($showVoteScreen);

}

function jal_democracy_archives($show_current = false, $before_title = false, $after_title = false)
{
	echo get_jal_democracy_archives($show_current, $before_title, $after_title);
}

function get_jal_democracy_archives($show_current = false, $before_title = false, $after_title = false)
{
	global $wpdb, $table_prefix;
	$where = ($show_current) ? '' : 'WHERE current = 0';
	$ids = $wpdb->get_results("SELECT id, current FROM {$table_prefix}democracyQ {$where} ORDER BY current DESC, id DESC");
	
	$output = "<div class='dem-archives'>";
	foreach ($ids as $id)
	{
		$democracy = new Democracy($id->id, $before_title, $after_title);
		$democracy->active = false;
		$output .= $democracy->display($id->current);
	}
	$output .= "</div>";
	
	return $output;
}

// user interaction
if (isset($_GET['dem_action']) && $ajax)
{

    header('Content-Type: text/html; charset='.get_option('blog_charset'));

	$id = (int) $_GET['dem_poll_id'];
	$democracy = new Democracy($id);
	switch ($_GET['dem_action']) 
	{
		// cast a vote, then see result
		case "vote" :
		
			$vote_id = (int) $_GET['dem_poll_'.$id];
			// poll id and whether to set cookie
			$democracy->addVote($vote_id);

			if (!isset($_GET['dem_ajax']) && isset($_SERVER['HTTP_REFERER']))
			{
				header('Location: '.$_SERVER['HTTP_REFERER']);
			}

			echo $democracy->showResults();

			break;
		
		// see result without voting
		case "view" :
		
			echo $democracy->showResults();

			break;


        case "show_vote_screen" :
            
            echo $democracy->showVoteScreen();
            
            break;

        // user is trying to add a new answer
		case "add_answer" :
		
			$aid = $democracy->addInlineAnswer($_GET['dem_new_answer']);

			$democracy->addVote($aid);

			if (!isset($_GET['dem_ajax']) && isset($_SERVER['HTTP_REFERER']))
			{
				
				$gets = array('dem_poll_id', 'dem_add_user_answer', 'dem_action');
				$url = $_SERVER['HTTP_REFERER'];
				
				if (isset($_GET[$gets[0]]) || isset($_GET[$gets[1]]) || isset($_GET[$gets[1]]))
				{
					foreach ($gets as $get)
						$url = remove_query_arg($get, $url);
				}
				
				header('Location: '.$url);
			}

			echo $democracy->showResults($aid);			
			
			break;
		

	}
}


// adds democracy db tables and initializes itself
function jal_install_democracy ()
{
	global $table_prefix, $wpdb;

	// never used Democracy before
    $first_time = $wpdb->get_var("SHOW TABLES LIKE '{$table_prefix}democracyQ'") != $table_prefix."democracyQ";

	// upgrading from 1.2
	$upgrading = !$first_time && $wpdb->get_var("SHOW TABLES LIKE '{$table_prefix}democracyIP'") != $table_prefix."democracyIP";

	if ($upgrading)
	{
		$questions = $wpdb->get_results("SELECT * FROM {$table_prefix}democracyQ");
		$answers = $wpdb->get_results("SELECT * FROM {$table_prefix}democracyA");
		
		$wpdb->query("DROP TABLE {$table_prefix}democracyQ");
		$wpdb->query("DROP TABLE {$table_prefix}democracyA");
	}
	
	
   $qry = "
CREATE TABLE {$table_prefix}democracyQ (
 id int(10) unsigned NOT NULL auto_increment,
 question text NOT NULL,
 added int(10) unsigned NOT NULL default '0',
 allowusers tinyint(1) unsigned NOT NULL default '0',
 current tinyint(1) unsigned NOT NULL default '0',
 active tinyint(1) unsigned NOT NULL default '0',
 multiple tinyint(1) unsigned NOT NULL default '0',
 PRIMARY KEY  (id),
 KEY current (current)
 );

CREATE TABLE {$table_prefix}democracyA (
  aid int(10) unsigned NOT NULL auto_increment,
  qid int(10) unsigned NOT NULL default '0',
  answer text NOT NULL,
  votes mediumint(6) unsigned NOT NULL default '0',
  added_by tinyint(1) unsigned NOT NULL default '0',
  PRIMARY KEY  (aid),
  KEY qid (qid)
);

CREATE TABLE {$table_prefix}democracyIP (
  ip int(11) NOT NULL default '0',
  qid int(10) unsigned NOT NULL default '0',
  KEY ip (ip,qid)
);
    ";
    
    require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
    dbDelta($qry);


    if ($first_time)
    {
        // create a sample, deactivated poll
        $time = time();
        $wpdb->query("INSERT INTO {$table_prefix}democracyQ VALUES (1, '".__('Rate my site', 'democracy')."', {$time}, 0, 0, 1, 0);");
        
        $choice[] = "(1, 1, '".__('Best. Blog. Ever.', 'democracy')."', 0, 0)";
        $choice[] = "(2, 1, '".__('Could be better&hellip;', 'democracy')."', 0, 0)";
        $choice[] = "(3, 1, '".__('My grandma could do better', 'democracy')."', 0, 0)";
        $choice[] = "(4, 1, '".__('Ooh look! A butterfly!', 'democracy')."', 0, 0)";
        $choice[] = "(5, 1, '".__('No Comment', 'democracy')."', 0, 0)";
        
        $values = implode(',', $choice);
        $wpdb->query("INSERT INTO {$table_prefix}democracyA VALUES {$values}");
	}
    
	if ($upgrading)
	{
		// import questions
		$inserts = array();
		$iplist = array();
		foreach ($questions as $question)
		{
			$iplist[$question->id] = unserialize($question->voters);
			$q = $wpdb->escape(stripslashes($question->question));
			$inserts[] = "({$question->id}, '{$q}', {$question->timestamp},  {$question->allowusers}, {$question->active}, 1, 0)";
		}

		$insert = implode(',', $inserts);
		$wpdb->query("INSERT INTO {$table_prefix}democracyQ VALUES {$insert}");

		// import answers
		$inserts = array();
		foreach ($answers as $answer)
		{
			$ans = $wpdb->escape(stripslashes($answer->answers));
			$inserts[] = "({$answer->aid}, {$answer->qid}, '{$ans}', {$answer->votes}, {$answer->added_by})";
		}

		$insert = implode(',', $inserts);
		$wpdb->query("INSERT INTO {$table_prefix}democracyA VALUES {$insert}");


		// import IPs
		$inserts = array();
		foreach ($iplist as $poll_id => $ips)
		{
			if ($ips)
				foreach ($ips as $ip)
					$inserts[] = "(".ip2long($ip).", {$poll_id})";
		}

		$insert = implode(',', $inserts);
		$wpdb->query("INSERT INTO {$table_prefix}democracyIP VALUES {$insert}");		
	}


	/***
	Add default options
	If they've already installed, their options won't be overwritten
	***/
        
    // check user's IP before their vote is counted
    add_option('democracy_logIPs', '1');
    // Graph percentages as a percent of the total votes or of the winner
    // by default,  the winning vote will be 100% of the graph, and the other answers will be a percent of that winner
    add_option('democracy_graph_from_total', '0');
    // store the user's cookie for one year
    add_option('democracy_cookie_days','365');
    // sort the answers by most votes = FALSE
    add_option('democracy_order_answers', '0');
    // HTML that begins the wraps around the title
    add_option('democracy_before_title','<strong class="poll-question">');
    // HTML that ends the wraps around the title
    add_option('democracy_after_title','</strong>');

	// add capability to use admin panel for editors and admins
	// if using the Role Manager plugin, this can be editable
	global $wp_roles;
	// only add roles if they don't exist yet
	if (!isset($wp_roles->roles['administrator']['capabilities']['democracy_admin']))
	{
		$wp_roles->add_cap('editor', 'democracy_admin');
		$wp_roles->add_cap('administrator', 'democracy_admin');   
	} 
}


// Adds the Democracy Poll Plugin tab to the admin navigation
function jal_add_page()
{	
	if (current_user_can('democracy_admin'))
	{
		/*  not a top-level..
    	add_menu_page(__('Democracy', 'democracy'), __('Democracy', 'democracy'), 1, 'democracy', 'jal_dem_admin_page');
        add_submenu_page('democracy', 'Manage', 'Manage', 1, 'democracy');
		*/
		
        add_submenu_page('edit.php', __('Democracy Poll', 'democracy'), __('Democracy Poll', 'democracy'), 1, 'democracy', 'jal_dem_admin_page');

    }
}


// shows the management html
function jal_dem_admin_page ()
{
	jal_dem_check_admin();
    include ABSPATH.'/wp-content/plugins/democracy/admin.php';
}

// adds the javascript for management page
function jal_dem_admin_head()
{
	global $jal_dem_version;
    $url = get_bloginfo('wpurl');
    echo "\n\t<!-- Added By Democracy Plugin. Version {$jal_dem_version} -->";
    echo "\n\t<script type='text/javascript' src='{$url}/wp-content/plugins/democracy/admin.js'></script>";
    echo "\n\t<link rel='stylesheet' href='{$url}/wp-content/plugins/democracy/admin.css' type='text/css' />\n";
}


// adds the styling and javascript to the WP head
function jal_add_js ()
{

    global $jal_dem_version;

    // if you're having problems with ajax loading, switch the commented lines below

    //$jal_wp_url = (dirname($_SERVER['PHP_SELF']) == "/") ? "/" : dirname($_SERVER['PHP_SELF']) . "/";
    $jal_wp_url = get_bloginfo('wpurl') . "/";

    echo "\n\t<!-- Added By Democracy Plugin. Version {$jal_dem_version} -->";
    echo "\n\t<script type='text/javascript' src='{$jal_wp_url}wp-content/plugins/democracy/democracy.js'></script>";
    echo "\n\t<link rel='stylesheet' href='{$jal_wp_url}wp-content/plugins/democracy/basic.css' type='text/css' />";
    echo "\n\t<link rel='stylesheet' href='{$jal_wp_url}wp-content/plugins/democracy/style.css' type='text/css' />\n";
}

// only added if they're on a dupe page via 'add an answer' or 'view results'
function jal_dem_noindex()
{
	echo "\n\t<!-- Added by democracy because this page is duplicate content (just with view results on) -->";
	echo "\n\t<meta name='robots' content='none' />\n";
}

// add <div> wrapper to {democracy} so that the HTML will stay valid
function jal_add_dem_div($content)
{
	return preg_replace('/{democracy[\w\W\s^}]*?}/', '\n<div>\\0</div>', $content);
}


// remove <div>s wrapping democracy when editing to make process transparent to user
function jal_remove_dem_div($content)
{
	return preg_replace('/\n(&lt;div&gt;|<div>)({democracy[\w\W\s^}]*?})(&lt;\/div&gt;|<\/div>)/', '\\2', $content);
}

// renders template tags within posts: {democracy:#}
function jal_insert_poll($content)
{
	// cut out early if they don't have democracy
	if (strpos($content, '{democracy') === false)
		return $content;
	
	// just get the standard poll
	if (strpos($content, '{democracy}') !== false)
	{
		$poll = new Democracy();
		$html = $poll->display($showVoteScreen = !(isset($_GET['dem_action']) && $_GET['dem_action'] == 'view' && ($_GET['dem_poll_id'] == $poll->id || $poll->isCurrent)));
		$content = str_replace('{democracy}', $html, $content);
	}
	
	// the archives page
	if (strpos($content, '{democracy_archives}') !== false)
	{
		$content = str_replace('{democracy_archives}', get_jal_democracy_archives(), $content);
	}

	// cut out again if they don't have democracy anymore
	if (strpos($content, '{democracy') === false)
		return $content;

	// find all instances of ids
	preg_match_all('/{democracy:([0-9]*)}/i', $content, $txt);
	$ids = array_unique($txt[1]);
		
	foreach ($ids as $id)
	{
		$poll = new Democracy($id);
		$html = $poll->display($showVoteScreen = !(isset($_GET['dem_action']) && $_GET['dem_action'] == 'view' && ($_GET['dem_poll_id'] == $poll->id || $poll->isCurrent)));
		$content = str_replace('{democracy:'.$id.'}', $html, $content);
	}
	
	return $content;
}

function jal_delete_poll()
{
    jal_dem_check_admin();
    global $wpdb, $table_prefix;

    $id = (int) $_GET['dem_poll_id'];

    // Delete the poll question and its answers
    $wpdb->query("DELETE FROM {$table_prefix}democracyQ WHERE id = {$id} LIMIT 1");
    $wpdb->query("DELETE FROM {$table_prefix}democracyA WHERE qid = {$id}");
    $wpdb->query("DELETE FROM {$table_prefix}democracyIP WHERE qid = {$id}");
}

function jal_activate_poll () 
{
    jal_dem_check_admin();
    global $wpdb, $table_prefix;

    $id = (int) $_GET['dem_poll_id'];
	$wpdb->query("UPDATE {$table_prefix}democracyQ SET current = 0");
	$wpdb->query("UPDATE {$table_prefix}democracyQ SET current = 1 WHERE id = {$id} LIMIT 1");
}

function jal_deactivate_poll () 
{
    jal_dem_check_admin();
    global $wpdb, $table_prefix;

    $id = (int) $_GET['dem_poll_id'];
	$wpdb->query("UPDATE {$table_prefix}democracyQ SET current = 0 WHERE id = {$id} LIMIT 1");
}


function jal_create_poll ()
{

	jal_dem_check_admin();
	global $wpdb, $table_prefix;
	
	$question = $wpdb->escape(trim($_POST['jal_dem_question']));
	$added = time();
	
	// options
	$allowusers = (int) isset($_POST['jal_dem_allow_users']);
	$current = (int) isset($_POST['jal_dem_current_poll']);
	$multiple = (int) isset($_POST['jal_dem_multiple']);
	// poll is active to be answered by default
	$active = 1;
	
	if ($current)
		$wpdb->query("UPDATE {$table_prefix}democracyQ SET current = 0");
	
	$wpdb->query(
		"INSERT INTO {$table_prefix}democracyQ ".
		"VALUES('', '{$question}', {$added}, {$allowusers}, {$current}, {$active}, {$multiple})"
	);
	
	$poll_id = $wpdb->insert_id;
	$answers = $_POST['jal_dem_answers'];
	
	foreach ($answers as $answer)
	{
		$answer = $wpdb->escape(trim($answer));
		
		// aid, qid, answer, votes, added_by
		if (!empty($answer))
			$sql_answers[] = "('', {$poll_id}, '{$answer}', 0, 0)";
	
	}
	
	if (!empty($sql_answers))
	{
		$values = implode(',', $sql_answers);
		$wpdb->query("INSERT INTO {$table_prefix}democracyA VALUES {$values}");
	}

}

function jal_edit_poll()
{
	jal_dem_check_admin();
	global $wpdb, $table_prefix;
	
	$poll_id = (int) $_POST['dem_poll_id'];
	
	$question = $wpdb->escape($_POST['jal_dem_question']);
	
	$current = (isset($_POST['jal_dem_current_poll'])) ? 1 : 0;
	$multiple = (isset($_POST['jal_dem_multiple'])) ? 1 : 0;
	$allow_users = (isset($_POST['jal_dem_allow_users'])) ? 1 : 0;
	
	$wpdb->query("UPDATE {$table_prefix}democracyQ SET question = '{$question}', allowusers = {$allow_users}, current = {$current}, multiple = {$multiple} WHERE id = {$poll_id}");

	$still_there = $_POST['jal_dem_old_answer'];
	// array of answer ids for answers that don't need to be updated
	$no_update = array();
	// array of answer ids for answers that were deleted
	$delete = array();
	
	$old_answers = $wpdb->get_results("SELECT * FROM {$table_prefix}democracyA WHERE qid = {$poll_id}");
	foreach ($old_answers as $answer)
	{
		if (isset($still_there[$answer->aid]))
		{
			if (stripslashes($answer->answer) == $still_there[$answer->aid])
				$no_update[] = $answer->aid;
				
			else if (trim($still_there[$answer->aid]) == '')
				$delete[] = $answer->aid;
		}
		else
		{
			$delete[] = $answer->aid;
		}
	}
	
	if (!empty($delete))
	{
		$ids = implode(',', $delete);
		$wpdb->query("DELETE FROM {$table_prefix}democracyA WHERE qid = {$poll_id} AND aid IN ({$ids}) LIMIT ".count($delete));
	}

	// Update old answers	
	foreach ($still_there as $aid => $newValue)
	{
		$aid = (int) $aid;
		if (in_array($aid, $no_update))
			continue;
		
		$newValue = $wpdb->escape($newValue);
		$wpdb->query("UPDATE {$table_prefix}democracyA SET answer = '{$newValue}' WHERE qid = {$poll_id} AND aid = {$aid} LIMIT 1");
	}
	
	// Add new answers
	$inserts = array();
	if (isset($_POST['jal_dem_answers']))
	{
		foreach ($_POST['jal_dem_answers'] as $newValue)
		{
			$answer = $wpdb->escape($newValue);
			$inserts[] = "(null, {$poll_id}, '{$answer}', 0, 0)";
		}
		
		$insert = implode(',', $inserts);
		$wpdb->query("INSERT INTO {$table_prefix}democracyA VALUES {$insert}");
	}	
	
}

function jal_update_poll_options()
{
	jal_dem_check_admin();

	$log_ip = (int) isset($_POST['democracy_logIPs']);
	update_option('democracy_logIPs', $log_ip);

	$graph = (int) isset($_POST['democracy_graph_from_total']);
	update_option('democracy_graph_from_total', $graph);

	$cookie_days = (int) $_POST['democracy_cookie_days'];
	update_option('democracy_cookie_days', $cookie_days);

	$order = (int) isset($_POST['democracy_order_answers']);
	update_option('democracy_order_answers', $order);
		
	$before = $_POST['democracy_before_title'];
	update_option('democracy_before_title', $before);

	$after = $_POST['democracy_after_title'];
	update_option('democracy_after_title', $after);

}

function jal_dem_check_admin() 
{
	if (!current_user_can('democracy_admin'))
    {
    	echo "\n\t<div class='error'>\n\t\t<p>";
        	_e('You do not have permission to access the Democracy admin panel.', 'democracy');
    	echo "</p>\n\t</div>";
    	die;
    }
}


/*========================*/
// The following make democracy plug-n-play
// by hooking into its various actions
/*========================*/

// installing democracy
add_action('activate_democracy/democracy.php', 'jal_install_democracy');

// javascript for main blog
add_action('wp_head', 'jal_add_js');
 // add the management page to the admin nav bar
add_action('admin_menu', 'jal_add_page');
// add javascript to admin area, only on democracy admin page
if ($_REQUEST['page'] == "democracy")
    add_action('admin_head', 'jal_dem_admin_head');

// Add a new question and its answers via admin panel
if (isset($_POST['jal_create_poll']))
    add_action('init', 'jal_create_poll');

// Edit a question and its answers via admin panel
if (isset($_POST['jal_edit_poll']))
    add_action('init', 'jal_edit_poll');

// When user deletes a poll
if (isset($_GET['jal_dem_delete']))
    add_action('init', 'jal_delete_poll');

// When user activates a poll
if (isset($_GET['jal_dem_activate']))
    add_action('init', 'jal_activate_poll');

// When user deactivates a poll
if (isset($_GET['jal_dem_deactivate']))
    add_action('init', 'jal_deactivate_poll');

if (isset($_POST['jal_dem_gen_options']))
	add_action('init', 'jal_update_poll_options');

// check post content in both posts and pages for {democracy}
add_filter('the_content', 'jal_insert_poll');

// add <div> wrapper to {democracy} so that the HTML will stay valid
add_filter('content_save_pre', 'jal_add_dem_div');

// remove <div>s wrapping democracy when editing to make process transparent to user
add_filter('content_edit_pre', 'jal_remove_dem_div', 1);


$gets = array('dem_poll_id', 'dem_add_user_answer', 'dem_action');
if (isset($_GET[$gets[0]]) || isset($_GET[$gets[1]]) || isset($_GET[$gets[1]]))
{
	add_filter('wp_head', 'jal_dem_noindex');
}

?>